@@ -1,6 +1,8 @@
 use std::fmt;
 use std::io::{self, Read, Seek, SeekFrom, Write};
 
+use crate::{Img, ImgBuf, ImgFmt, ImgMut, ImgRef};
+
 // [Table 3-5 Well-known data types](https://developer.apple.com/library/archive/documentation/QuickTime/QTFF/Metadata/Metadata.html#//apple_ref/doc/uid/TP40000939-CH1-SW34) codes
 /// Reserved for use where no type needs to be indicated.
 pub(crate) const RESERVED: u32 = 0;
@@ -115,8 +117,18 @@ impl fmt::Debug for Data {
     }
 }
 
+impl<T: Into<Vec<u8>>> From<Img<T>> for Data {
+    fn from(image: Img<T>) -> Self {
+        match image.fmt {
+            ImgFmt::Bmp => Self::Bmp(image.data.into()),
+            ImgFmt::Jpeg => Self::Jpeg(image.data.into()),
+            ImgFmt::Png => Self::Png(image.data.into()),
+        }
+    }
+}
+
 impl Data {
-    /// Returns the length in bytes.
+    /// Returns the length of the raw data in bytes.
     pub fn len(&self) -> u64 {
         (match self {
             Self::Reserved(v) => v.len(),
@@ -129,7 +141,7 @@ impl Data {
         }) as u64
     }
 
-    /// Returns true if the data is empty, false otherwise.
+    /// Returns true if the data is of length 0, false otherwise.
     pub fn is_empty(&self) -> bool {
         self.len() == 0
     }
@@ -144,7 +156,8 @@ impl Data {
         matches!(self, Self::Utf8(_) | Self::Utf16(_))
     }
 
-    /// Returns true if `self` is of type [`Self::Jpeg`], [`Self::Png`] or [`Self::Bmp`] false otherwise.
+    /// Returns true if `self` is of type [`Self::Jpeg`], [`Self::Png`] or [`Self::Bmp`] false
+    /// otherwise.
     pub const fn is_image(&self) -> bool {
         matches!(self, Self::Jpeg(_) | Self::Png(_) | Self::Bmp(_))
     }
@@ -184,8 +197,9 @@ impl Data {
         matches!(self, Self::Bmp(_))
     }
 
-    /// Returns a byte vec reference if `self` is of type [`Self::Reserved`] or [`Self::BeSigned`].
-    pub const fn bytes(&self) -> Option<&Vec<u8>> {
+    /// Returns a reference to byte data if `self` is of type [`Self::Reserved`] or
+    /// [`Self::BeSigned`].
+    pub fn bytes(&self) -> Option<&[u8]> {
         match self {
             Self::Reserved(v) => Some(v),
             Self::BeSigned(v) => Some(v),
@@ -193,7 +207,7 @@ impl Data {
         }
     }
 
-    /// Returns a mutable byte vec reference if `self` is of type [`Self::Reserved`] or
+    /// Returns a mutable reference to byte data if `self` is of type [`Self::Reserved`] or
     /// [`Self::BeSigned`].
     pub fn bytes_mut(&mut self) -> Option<&mut Vec<u8>> {
         match self {
@@ -203,9 +217,9 @@ impl Data {
         }
     }
 
-    /// Consumes `self` and returns a byte vec if `self` is of type [`Self::Reserved`] or
+    /// Consumes `self` and returns byte data if `self` is of type [`Self::Reserved`] or
     /// [`Self::BeSigned`].
-    pub fn take_bytes(self) -> Option<Vec<u8>> {
+    pub fn into_bytes(self) -> Option<Vec<u8>> {
         match self {
             Self::Reserved(v) => Some(v),
             Self::BeSigned(v) => Some(v),
@@ -213,7 +227,7 @@ impl Data {
         }
     }
 
-    /// Returns a string reference if `self` is either of type [`Self::Utf8`] or [`Self::Utf16`].
+    /// Returns a reference to a string if `self` is of type [`Self::Utf8`] or [`Self::Utf16`].
     pub fn string(&self) -> Option<&str> {
         match self {
             Self::Utf8(s) => Some(s.as_str()),
@@ -222,7 +236,7 @@ impl Data {
         }
     }
 
-    /// Returns a mutable string reference if `self` is either of type [`Self::Utf8`] or
+    /// Returns a mutable reference to a string if `self` is of type [`Self::Utf8`] or
     /// [`Self::Utf16`].
     pub fn string_mut(&mut self) -> Option<&mut String> {
         match self {
@@ -232,9 +246,8 @@ impl Data {
         }
     }
 
-    /// Consumes `self` and returns a string if `self` is either of type [`Self::Utf8`] or
-    /// [`Self::Utf16`].
-    pub fn take_string(self) -> Option<String> {
+    /// Consumes `self` and returns a string if `self` is of type [`Self::Utf8`] or [`Self::Utf16`].
+    pub fn into_string(self) -> Option<String> {
         match self {
             Self::Utf8(s) => Some(s),
             Self::Utf16(s) => Some(s),
@@ -242,130 +255,115 @@ impl Data {
         }
     }
 
-    /// Returns a data reference if `self` is of type [`Self::Jpeg`], [`Self::Png`] or
+    /// Returns a reference to an image if `self` is of type [`Self::Jpeg`], [`Self::Png`] or
     /// [`Self::Bmp`].
-    pub const fn image(&self) -> Option<&Data> {
+    pub fn image(&self) -> Option<ImgRef> {
         match self {
-            d if d.is_jpeg() => Some(d),
-            d if d.is_png() => Some(d),
-            d if d.is_bmp() => Some(d),
+            Self::Jpeg(v) => Some(Img::new(ImgFmt::Jpeg, v)),
+            Self::Png(v) => Some(Img::new(ImgFmt::Png, v)),
+            Self::Bmp(v) => Some(Img::new(ImgFmt::Bmp, v)),
             _ => None,
         }
     }
 
-    /// Returns a data reference if `self` is of type [`Self::Jpeg`], [`Self::Png`] or
-    /// [`Self::Bmp`].
-    pub fn image_mut(&mut self) -> Option<&mut Data> {
-        match self {
-            d if d.is_jpeg() => Some(d),
-            d if d.is_png() => Some(d),
-            d if d.is_bmp() => Some(d),
-            _ => None,
-        }
-    }
-
-    /// Consumes `self` and returns data if `self` is of type [`Self::Jpeg`], [`Self::Png`] or
-    /// [`Self::Bmp`].
-    pub fn take_image(self) -> Option<Data> {
-        match self {
-            d if d.is_jpeg() => Some(d),
-            d if d.is_png() => Some(d),
-            d if d.is_bmp() => Some(d),
-            _ => None,
-        }
-    }
-
-    /// Returns a byte vec reference if `self` is of type [`Self::Jpeg`], [`Self::Png`] or
-    /// [`Self::Bmp`].
-    pub const fn image_data(&self) -> Option<&Vec<u8>> {
-        match self {
-            Self::Jpeg(v) => Some(v),
-            Self::Png(v) => Some(v),
-            Self::Bmp(v) => Some(v),
-            _ => None,
-        }
-    }
-
-    /// Returns a mutable byte vec reference if `self` is of type [`Self::Jpeg`], [`Self::Png`] or
-    /// [`Self::Bmp`].
-    pub fn image_data_mut(&mut self) -> Option<&mut Vec<u8>> {
-        match self {
-            Self::Jpeg(v) => Some(v),
-            Self::Png(v) => Some(v),
-            Self::Bmp(v) => Some(v),
-            _ => None,
-        }
-    }
-
-    /// Consumes `self` and returns a byte vec if `self` is of type [`Self::Jpeg`], [`Self::Png`]
+    /// Returns a mutable reference to an image if `self` is of type [`Self::Jpeg`], [`Self::Png`]
     /// or [`Self::Bmp`].
-    pub fn take_image_data(self) -> Option<Vec<u8>> {
+    pub fn image_mut(&mut self) -> Option<ImgMut> {
         match self {
-            Self::Jpeg(v) => Some(v),
-            Self::Png(v) => Some(v),
-            Self::Bmp(v) => Some(v),
+            Self::Jpeg(v) => Some(Img::new(ImgFmt::Jpeg, v)),
+            Self::Png(v) => Some(Img::new(ImgFmt::Png, v)),
+            Self::Bmp(v) => Some(Img::new(ImgFmt::Bmp, v)),
             _ => None,
         }
     }
 
-    /// Returns a byte vec reference if `self` is of type [`Self::Reserved`].
-    pub const fn reserved(&self) -> Option<&Vec<u8>> {
+    /// Consumes `self` and returns an image if `self` is of type [`Self::Jpeg`], [`Self::Png`] or
+    /// [`Self::Bmp`].
+    pub fn into_image(self) -> Option<ImgBuf> {
+        match self {
+            Self::Jpeg(v) => Some(Img::new(ImgFmt::Jpeg, v)),
+            Self::Png(v) => Some(Img::new(ImgFmt::Png, v)),
+            Self::Bmp(v) => Some(Img::new(ImgFmt::Bmp, v)),
+            _ => None,
+        }
+    }
+
+    /// Returns a reference to image data if `self` is of type [`Self::Jpeg`], [`Self::Png`] or
+    /// [`Self::Bmp`].
+    pub fn image_data(&self) -> Option<&[u8]> {
+        self.image().map(|i| i.data)
+    }
+
+    /// Returns a mutable reference to image data if `self` is of type [`Self::Jpeg`], [`Self::Png`]
+    /// or [`Self::Bmp`].
+    pub fn image_data_mut(&mut self) -> Option<&mut Vec<u8>> {
+        self.image_mut().map(|i| i.data)
+    }
+
+    /// Consumes `self` and returns image data if `self` is of type [`Self::Jpeg`], [`Self::Png`]
+    /// or [`Self::Bmp`].
+    pub fn into_image_data(self) -> Option<Vec<u8>> {
+        self.into_image().map(|i| i.data)
+    }
+
+    /// Returns a reference to byte data if `self` is of type [`Self::Reserved`].
+    pub fn reserved(&self) -> Option<&[u8]> {
         match self {
             Self::Reserved(v) => Some(v),
             _ => None,
         }
     }
 
-    /// Returns a string reference if `self` is of type [`Self::Utf8`].
-    pub const fn utf8(&self) -> Option<&String> {
+    /// Returns a reference to a string if `self` is of type [`Self::Utf8`].
+    pub fn utf8(&self) -> Option<&str> {
         match self {
             Self::Utf8(s) => Some(s),
             _ => None,
         }
     }
 
-    /// Returns a string reference if `self` is of type [`Self::Utf16`].
-    pub const fn utf16(&self) -> Option<&String> {
+    /// Returns a reference to a string if `self` is of type [`Self::Utf16`].
+    pub fn utf16(&self) -> Option<&str> {
         match self {
             Self::Utf16(s) => Some(s),
             _ => None,
         }
     }
 
-    /// Returns a byte vec reference if `self` is of type [`Self::Jpeg`].
-    pub const fn jpeg(&self) -> Option<&Vec<u8>> {
+    /// Returns a reference to image data if `self` is of type [`Self::Jpeg`].
+    pub fn jpeg(&self) -> Option<&[u8]> {
         match self {
             Self::Jpeg(v) => Some(v),
             _ => None,
         }
     }
 
-    /// Returns a byte vec reference if `self` is of type [`Self::Png`].
-    pub const fn png(&self) -> Option<&Vec<u8>> {
+    /// Returns a reference to image data if `self` is of type [`Self::Png`].
+    pub fn png(&self) -> Option<&[u8]> {
         match self {
             Self::Png(v) => Some(v),
             _ => None,
         }
     }
 
-    /// Returns a byte vec reference if `self` is of type [`Self::BeSigned`].
-    pub const fn be_signed(&self) -> Option<&Vec<u8>> {
+    /// Returns a reference to byte data if `self` is of type [`Self::BeSigned`].
+    pub fn be_signed(&self) -> Option<&[u8]> {
         match self {
             Self::BeSigned(v) => Some(v),
             _ => None,
         }
     }
 
-    /// Returns a byte vec reference if `self` is of type [`Self::Bmp`].
-    pub const fn bmp(&self) -> Option<&Vec<u8>> {
+    /// Returns a reference to byte data if `self` is of type [`Self::Bmp`].
+    pub fn bmp(&self) -> Option<&[u8]> {
         match self {
             Self::Bmp(v) => Some(v),
             _ => None,
         }
     }
 
-    /// Attempts to write the typed data to the writer.
-    pub fn write_typed(&self, writer: &mut impl Write) -> crate::Result<()> {
+    /// Attempts to write type header followed by the data to the writer.
+    pub(crate) fn write_typed(&self, writer: &mut impl Write) -> crate::Result<()> {
         let datatype = match self {
             Self::Reserved(_) => RESERVED,
             Self::Utf8(_) => UTF8,
@@ -386,7 +384,7 @@ impl Data {
     }
 
     /// Attempts to write the raw data to the writer.
-    pub fn write_raw(&self, writer: &mut impl Write) -> crate::Result<()> {
+    pub(crate) fn write_raw(&self, writer: &mut impl Write) -> crate::Result<()> {
         match self {
             Self::Reserved(v) => {
                 writer.write_all(v)?;
@@ -504,13 +502,13 @@ pub(crate) fn remaining_stream_len(reader: &mut impl Seek) -> io::Result<u64> {
 ///
 /// # Example
 /// ```
-/// # #[macro_use] extern crate mp4ameta; fn main() {
-/// let bytes = vec![0u8, 0, 0, 0, 0, 0, 1, 3];
+/// # #[macro_use] extern crate mp4ameta;
+/// # fn main() {
+/// let bytes = vec![0x00, 0x00, 0x00, 0x00, 0x2D, 0x34, 0xD0, 0x5E];
 /// let int = be_int!(bytes, 4, u32);
-/// assert_eq!(int, Some(259u32));
+/// assert_eq!(int, Some(758435934u32));
 /// # }
 /// ```
-#[macro_export]
 macro_rules! be_int {
     ($bytes:expr, $index:expr, $type:ty) => {{
         use std::convert::TryFrom;
@@ -543,7 +541,6 @@ macro_rules! be_int {
 /// assert_eq!(bytes[5], 12u8);
 /// # }
 /// ```
-#[macro_export]
 macro_rules! set_be_int {
     ($bytes:expr, $index:expr, $value:expr, $type:ty) => {{
         const SIZE: usize = std::mem::size_of::<$type>();
