@@ -1,10 +1,9 @@
+use std::fs;
+use std::path::{Path, PathBuf};
+use std::time::Duration;
+
 use mp4ameta::{
-    AdvisoryRating, ChannelConfig, Data, Fourcc, MediaType, SampleRate, Tag, STANDARD_GENRES,
-};
-use std::{
-    fs,
-    path::{Path, PathBuf},
-    time::Duration,
+    AdvisoryRating, ChannelConfig, Data, Fourcc, Img, MediaType, SampleRate, Tag, STANDARD_GENRES,
 };
 use walkdir::WalkDir;
 
@@ -40,7 +39,7 @@ fn read_dir(path: &str, fun: impl Fn(&Path, &Tag)) {
 
 fn get_tag_1() -> Tag {
     let mut tag = Tag::default();
-    tag.set_advisory_rating(AdvisoryRating::Explicit(4));
+    tag.set_advisory_rating(AdvisoryRating::Explicit);
     tag.set_album("TEST ALBUM");
     tag.set_album_artist("TEST ALBUM ARTIST");
     tag.set_artist("TEST ARTIST");
@@ -62,7 +61,7 @@ fn get_tag_1() -> Tag {
     tag.set_title("TEST TITLE");
     tag.set_track(7, 13);
     tag.set_year("2013");
-    tag.set_artwork(Data::Png(fs::read("files/artwork.png").unwrap()));
+    tag.set_artwork(Img::png(fs::read("files/artwork.png").unwrap()));
     tag.set_isrc("TEST ISRC");
     tag.set_lyricist("TEST LYRICIST");
     tag
@@ -92,14 +91,14 @@ fn get_tag_2() -> Tag {
     tag.set_title("NEW TITLE");
     tag.set_track(3, 7);
     tag.set_year("1998");
-    tag.set_artwork(Data::Jpeg(b"NEW ARTWORK".to_vec()));
+    tag.set_artwork(Img::jpeg(b"NEW ARTWORK".to_vec()));
     tag.set_isrc("NEW ISRC");
     tag.set_lyricist("NEW LYRICIST");
     tag
 }
 
 fn assert_tag_1(tag: &Tag) {
-    assert_eq!(tag.advisory_rating(), Some(AdvisoryRating::Explicit(4)));
+    assert_eq!(tag.advisory_rating(), Some(AdvisoryRating::Explicit));
     assert_eq!(tag.album(), Some("TEST ALBUM"));
     assert_eq!(tag.album_artist(), Some("TEST ALBUM ARTIST"));
     assert_eq!(tag.artist(), Some("TEST ARTIST"));
@@ -125,7 +124,7 @@ fn assert_tag_1(tag: &Tag) {
     assert_eq!(tag.track_number(), Some(7));
     assert_eq!(tag.total_tracks(), Some(13));
     assert_eq!(tag.year(), Some("2013"));
-    assert_eq!(tag.artwork(), Some(&Data::Png(fs::read("files/artwork.png").unwrap())));
+    assert_eq!(tag.artwork(), Some(Img::png(fs::read("files/artwork.png").unwrap().as_slice())));
     assert_eq!(tag.isrc(), Some("TEST ISRC"));
     assert_eq!(tag.lyricist(), Some("TEST LYRICIST"));
 }
@@ -157,13 +156,13 @@ fn assert_tag_2(tag: &Tag) {
     assert_eq!(tag.track_number(), Some(3));
     assert_eq!(tag.total_tracks(), Some(7));
     assert_eq!(tag.year(), Some("1998"));
-    assert_eq!(tag.artwork(), Some(&Data::Jpeg(b"NEW ARTWORK".to_vec())));
+    assert_eq!(tag.artwork(), Some(Img::jpeg(&b"NEW ARTWORK"[..])));
     assert_eq!(tag.isrc(), Some("NEW ISRC"));
     assert_eq!(tag.lyricist(), Some("NEW LYRICIST"));
 }
 
 fn assert_tag_3(tag: &Tag) {
-    assert_eq!(tag.advisory_rating(), Some(AdvisoryRating::Explicit(4)));
+    assert_eq!(tag.advisory_rating(), Some(AdvisoryRating::Explicit));
     assert_eq!(tag.album(), Some("TEST ALBUM"));
     assert_eq!(tag.album_artist(), Some("TEST ALBUM ARTIST"));
 
@@ -202,7 +201,7 @@ fn assert_tag_3(tag: &Tag) {
     assert_eq!(tag.track_number(), Some(7));
     assert_eq!(tag.total_tracks(), Some(13));
     assert_eq!(tag.year(), Some("2013"));
-    assert_eq!(tag.artwork(), Some(&Data::Png(fs::read("files/artwork.png").unwrap())));
+    assert_eq!(tag.artwork(), Some(Img::png(fs::read("files/artwork.png").unwrap().as_slice())));
     assert_eq!(tag.isrc(), Some("TEST ISRC"));
 
     let mut lyricists = tag.lyricists();
@@ -250,22 +249,22 @@ fn sample_files() {
         Tag::default().write_to_path(&path).unwrap();
         println!("reading empty tag");
         let tag = Tag::read_from_path(&path).unwrap();
-        assert!(&tag.atoms.is_empty());
-        assert_eq!(&tag.info, &t.info);
+        assert!(tag.is_empty());
+        assert_eq!(tag.audio_info(), t.audio_info());
 
         println!("writing sample tag 1");
         get_tag_1().write_to_path(&path).unwrap();
         println!("reading sample tag 1");
         let tag = Tag::read_from_path(&path).unwrap();
         assert_tag_1(&tag);
-        assert_eq!(&tag.info, &t.info);
+        assert_eq!(tag.audio_info(), t.audio_info());
 
         println!("writing sample tag 2");
         get_tag_2().write_to_path(&path).unwrap();
         println!("reading sample tag 2");
         let tag = Tag::read_from_path(&path).unwrap();
         assert_tag_2(&tag);
-        assert_eq!(&tag.info, &t.info);
+        assert_eq!(tag.audio_info(), t.audio_info());
         println!();
     });
 }
@@ -352,7 +351,7 @@ fn write_empty() {
 
     println!("reading...");
     let tag = Tag::read_from_path("target/write_empty.m4a").unwrap();
-    assert!(tag.atoms.is_empty());
+    assert!(tag.is_empty());
     assert_readonly(&tag);
 }
 
@@ -430,17 +429,16 @@ fn multiple_values() {
 
 #[test]
 fn genre_handling() {
-    let (code, name) = (5, STANDARD_GENRES[4]);
-
     let mut tag = Tag::default();
     assert_eq!(tag.genre(), None);
     assert_eq!(tag.standard_genre(), None);
     assert_eq!(tag.custom_genre(), None);
 
-    tag.set_genre(name);
-    assert_eq!(tag.genre(), Some(name));
-    assert_eq!(tag.standard_genre(), Some(code));
-    assert_eq!(tag.custom_genre(), None);
+    let standard_name = STANDARD_GENRES[38];
+    tag.set_genre(standard_name);
+    assert_eq!(tag.genre(), Some(standard_name));
+    assert_eq!(tag.standard_genre(), None);
+    assert_eq!(tag.custom_genre(), Some(standard_name));
 
     tag.set_genre("CUSTOM GENRE");
     assert_eq!(tag.genre(), Some("CUSTOM GENRE"));
@@ -580,7 +578,7 @@ fn tag_destructuring() {
     tag.set_lyrics("TEST LYRICS");
     tag.set_title("TEST TITLE");
     tag.set_year("2013");
-    tag.set_artwork(Data::Png(b"TEST ARTWORK".to_vec()));
+    tag.set_artwork(Img::png(b"TEST ARTWORK".to_vec()));
 
     assert_eq!(tag.take_album(), Some("TEST ALBUM".to_string()));
     assert_eq!(tag.take_album_artist(), Some("TEST ALBUM ARTIST".to_string()));
@@ -597,7 +595,7 @@ fn tag_destructuring() {
     assert_eq!(tag.take_lyrics(), Some("TEST LYRICS".to_string()));
     assert_eq!(tag.take_title(), Some("TEST TITLE".to_string()));
     assert_eq!(tag.take_year(), Some("2013".to_string()));
-    assert_eq!(tag.take_artwork(), Some(Data::Png(b"TEST ARTWORK".to_vec())));
+    assert_eq!(tag.take_artwork(), Some(Img::png(b"TEST ARTWORK".to_vec())));
 
     assert_eq!(tag.album(), None);
     assert_eq!(tag.album_artist(), None);
